@@ -7,7 +7,8 @@ import gettext
 import logging
 import sys
 import locale
-import gtk
+
+from gi.repository import GdkPixbuf
 
 from tryton import __version__
 
@@ -47,7 +48,6 @@ class ConfigManager(object):
             'client.toolbar': 'default',
             'client.save_width_height': True,
             'client.save_tree_state': True,
-            'client.fast_tabbing': True,
             'client.spellcheck': False,
             'client.lang': locale.getdefaultlocale()[0],
             'client.language_direction': 'ltr',
@@ -55,10 +55,9 @@ class ConfigManager(object):
             'client.limit': 1000,
             'client.check_version': False,
             'client.bus_timeout': 10 * 60,
-            'icon.color': '#3465a4',
+            'icon.colors': '#3465a4,#555753,#cc0000',
             'image.max_size': 10 ** 6,
-            'roundup.url': 'http://bugs.tryton.org/',
-            'roundup.xmlrpc': 'roundup-xmlrpc.tryton.org',
+            'bug.url': 'https://bugs.tryton.org/',
             'download.url': 'https://downloads.tryton.org/',
             'download.frequency': 60 * 60 * 8,
             'menu.pane': 200,
@@ -169,5 +168,5 @@ if not os.path.isdir(PIXMAPS_DIR):
     PIXMAPS_DIR = pkg_resources.resource_filename(
         'tryton', 'data/pixmaps/tryton')
 
-TRYTON_ICON = gtk.gdk.pixbuf_new_from_file(
+TRYTON_ICON = GdkPixbuf.Pixbuf.new_from_file(
     os.path.join(PIXMAPS_DIR, 'tryton-icon.png'))
