@@ -843,7 +843,7 @@ class ReferenceField(Field):
                         pass
                 if '%s,%s' % (ref_model, ref_id) == self.get(record):
                     rec_name = record.value.get(
-                        self.name + '.', {}).get('rec_name', '')
+                        self.name + '.', {}).get('rec_name') or ''
                 else:
                     rec_name = ''
             record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
@@ -878,7 +878,7 @@ class ReferenceField(Field):
         elif ref_model:
             rec_name = ''
         else:
-            rec_name = str(ref_id)
+            rec_name = str(ref_id) if ref_id is not None else ''
         record.value[self.name] = ref_model, ref_id
         record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
 
@@ -906,10 +906,10 @@ class ReferenceField(Field):
         else:
             model = None
         screen_domain, attr_domain = self.domains_get(record)
+        screen_domain = filter_leaf(screen_domain, self.name, model)
         screen_domain = prepare_reference_domain(screen_domain, self.name)
         return concat(localize_domain(
-                filter_leaf(screen_domain, self.name, model),
-                strip_target=True), attr_domain)
+                screen_domain, strip_target=True), attr_domain)
 
     def get_models(self, record):
         screen_domain, attr_domain = self.domains_get(record)
