@@ -1,35 +1,39 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-import gtk
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import gettext
 
-from many2one import Many2One
-from tryton.common.selection import SelectionMixin, PopdownMixin
+from gi.repository import Gtk
+
+from .many2one import Many2One
+from tryton.common.selection import SelectionMixin, PopdownMixin, \
+        selection_shortcuts
 
 _ = gettext.gettext
 
 
 class Reference(Many2One, SelectionMixin, PopdownMixin):
 
-    def __init__(self, field_name, model_name, attrs=None):
-        super(Reference, self).__init__(field_name, model_name, attrs=attrs)
+    def __init__(self, view, attrs):
+        super(Reference, self).__init__(view, attrs)
 
-        self.widget_combo = gtk.ComboBoxEntry()
+        self.widget_combo = Gtk.ComboBox(has_entry=True)
         child = self.widget_combo.get_child()
-        child.set_editable(False)
-        child.connect('changed', self.sig_changed_combo)
-        self.widget.pack_start(self.widget_combo, expand=False, fill=True)
+        child.connect('activate', lambda *a: self._focus_out())
+        child.connect('focus-out-event', lambda *a: self._focus_out())
+        child.get_accessible().set_name(attrs.get('string', ''))
+        self.widget_combo.connect('changed', self.sig_changed_combo)
+        self.widget_combo.connect('move-active', self._move_active)
+        self.widget_combo.connect(
+            'scroll-event',
+            lambda c, e: c.stop_emission_by_name('scroll-event'))
+        selection_shortcuts(self.widget_combo)
 
-        self.widget.pack_start(gtk.Label('-'), expand=False, fill=False)
+        self.widget.pack_start(
+            self.widget_combo, expand=False, fill=True, padding=0)
 
         self.init_selection()
         self.set_popdown(self.selection, self.widget_combo)
 
-        self.widget.set_focus_chain([self.widget_combo, self.wid_text])
-
-    def grab_focus(self):
-        return self.widget_combo.grab_focus()
-
     def get_model(self):
         active = self.widget_combo.get_active()
         if active < 0:
@@ -44,14 +48,16 @@ class Reference(Many2One, SelectionMixin, PopdownMixin):
                 return model, name
         return '', ''
 
-    def _readonly_set(self, value):
-        super(Reference, self)._readonly_set(value)
-        if not value:
-            self.widget.set_focus_chain([self.widget_combo, self.wid_text])
+    def _move_active(self, combobox, scroll_type):
+        if not combobox.get_child().get_editable():
+            combobox.stop_emission_by_name('move-active')
 
     def _set_button_sensitive(self):
         super(Reference, self)._set_button_sensitive()
-        self.widget_combo.set_sensitive(not self._readonly)
+        self.widget_combo.get_child().set_editable(not self._readonly)
+        self.widget_combo.set_button_sensitivity(
+            Gtk.SensitivityType.OFF if self._readonly
+            else Gtk.SensitivityType.AUTO)
 
     @property
     def modified(self):
@@ -92,7 +98,6 @@ class Reference(Many2One, SelectionMixin, PopdownMixin):
         if not self.changed:
             return
         self.wid_text.set_text('')
-        self.wid_text.set_position(0)
         model = self.get_model()
         if model:
             value = (model, (-1, ''))
@@ -100,23 +105,23 @@ class Reference(Many2One, SelectionMixin, PopdownMixin):
             value = ('', '')
         self.field.set_client(self.record, value)
 
-    def set_value(self, record, field):
+    def set_value(self):
         if not self.get_model():
             value = self.wid_text.get_text()
             if not value:
-                field.set_client(record, None)
+                self.field.set_client(self.record, None)
             else:
-                field.set_client(record, ('', value))
+                self.field.set_client(self.record, ('', value))
                 return
         else:
             try:
-                model, name = field.get_client(record)
+                model, name = self.field.get_client(self.record)
             except (ValueError, TypeError):
                 model, name = self.get_empty_value()
             if (model != self.get_model()
                     or name != self.wid_text.get_text()):
-                field.set_client(record, None)
-                self.wid_text.set_text('')
+                self.field.set_client(self.record, None)
+                self.set_text(None)
 
     def set_text(self, value):
         if value:
@@ -124,16 +129,15 @@ class Reference(Many2One, SelectionMixin, PopdownMixin):
         else:
             model, value = None, None
         super(Reference, self).set_text(value)
-        child = self.widget_combo.get_child()
-        child.handler_block_by_func(self.sig_changed_combo)
+        self.widget_combo.handler_block_by_func(self.sig_changed_combo)
         if not self.set_popdown_value(self.widget_combo, model):
             text = self.get_inactive_selection(model)
             self.set_popdown(
                 self.selection[:] + [(model, text)], self.widget_combo)
             self.set_popdown_value(self.widget_combo, value)
-        child.handler_unblock_by_func(self.sig_changed_combo)
+        self.widget_combo.handler_unblock_by_func(self.sig_changed_combo)
 
-    def display(self, record, field):
-        self.update_selection(record, field)
+    def display(self):
+        self.update_selection(self.record, self.field)
         self.set_popdown(self.selection, self.widget_combo)
-        super(Reference, self).display(record, field)
+        super(Reference, self).display()
