@@ -2,11 +2,14 @@
 # this repository contains the full copyright notices and license terms.
 import csv
 import gettext
+import locale
+from decimal import Decimal
 
-import gtk
+from gi.repository import Gtk
 
 import tryton.common as common
 from tryton.common import RPCExecute, RPCException
+from tryton.common.datetime_ import date_parse
 from tryton.gui.window.win_csv import WinCSV
 
 _ = gettext.gettext
@@ -26,33 +29,37 @@ class WinImport(WinCSV):
         self.dialog.set_title(_('CSV Import: %s') % name)
 
     def add_buttons(self, box):
-        button_autodetect = gtk.Button(
-            _('_Auto-Detect'), stock=None, use_underline=True)
-        button_autodetect.set_alignment(0.0, 0.0)
+        button_autodetect = Gtk.Button(
+            label=_('_Auto-Detect'), stock=None, use_underline=True)
         button_autodetect.set_image(common.IconFactory.get_image(
-                'tryton-search', gtk.ICON_SIZE_BUTTON))
+                'tryton-search', Gtk.IconSize.BUTTON))
         button_autodetect.set_always_show_image(True)
         button_autodetect.connect_after('clicked', self.sig_autodetect)
-        box.pack_start(button_autodetect, False, False, 0)
+        box.pack_start(
+            button_autodetect, expand=False, fill=False, padding=0)
 
     def add_chooser(self, box):
-        hbox_csv_import = gtk.HBox()
-        box.pack_start(hbox_csv_import, False, True, 4)
-        label_csv_import = gtk.Label(_("File to Import:"))
+        hbox_csv_import = Gtk.HBox()
+        box.pack_start(hbox_csv_import, expand=False, fill=True, padding=4)
+        label_csv_import = Gtk.Label(label=_("File to Import:"))
         hbox_csv_import.pack_start(label_csv_import, False, False, 0)
-        self.import_csv_file = gtk.FileChooserButton(_("Open..."))
+        self.import_csv_file = Gtk.FileChooserButton(title=_("Open..."))
         label_csv_import.set_mnemonic_widget(self.import_csv_file)
-        hbox_csv_import.pack_start(self.import_csv_file, True, True, 0)
+        hbox_csv_import.pack_start(
+            self.import_csv_file, expand=True, fill=True, padding=0)
 
-    def add_csv_header_param(self, table):
-        label_csv_skip = gtk.Label(_('Lines to Skip:'))
-        label_csv_skip.set_alignment(1, 0.5)
-        table.attach(label_csv_skip, 2, 3, 1, 2)
+    def add_csv_header_param(self, box):
+        label_csv_skip = Gtk.Label(
+            label=_('Lines to Skip:'), halign=Gtk.Align.START)
+        box.pack_start(label_csv_skip, expand=False, fill=True, padding=0)
 
-        self.csv_skip = gtk.SpinButton()
-        self.csv_skip.configure(gtk.Adjustment(0, 0, 100, 1, 10), 1, 0)
+        self.csv_skip = Gtk.SpinButton()
+        self.csv_skip.configure(Gtk.Adjustment(
+                value=0, lower=0, upper=100,
+                step_increment=1, page_increment=10),
+            1, 0)
         label_csv_skip.set_mnemonic_widget(self.csv_skip)
-        table.attach(self.csv_skip, 3, 4, 1, 2)
+        box.pack_start(self.csv_skip, expand=False, fill=True, padding=0)
 
     def model_populate(self, fields, parent_node=None, prefix_field='',
             prefix_name=''):
@@ -60,7 +67,7 @@ class WinImport(WinCSV):
         fields_order.sort(
             key=lambda x: fields[x].get('string', ''), reverse=True)
         for field in fields_order:
-            if not fields[field].get('readonly', False):
+            if not fields[field].get('readonly', False) or field == 'id':
                 self.fields_data[prefix_field + field] = fields[field]
                 name = fields[field]['string'] or field
                 node = self.model1.insert(
@@ -125,12 +132,9 @@ class WinImport(WinCSV):
                                     (prefix + parent):
                                 self.on_row_expanded(self.view1, iter,
                                         self.model1.get_path(iter))
-                                iter = self.model1.iter_children(iter)
-                                prefix = parent + '/'
                                 break
-                            else:
-                                iter = self.model1.iter_next(iter)
-
+                            iter = self.model1.iter_next(iter)
+                        prefix = parent + '/'
                 if word in self.fields_invert:
                     name = word
                     field = self.fields_invert[word]
@@ -166,7 +170,7 @@ class WinImport(WinCSV):
         self.model2.clear()
 
     def response(self, dialog, response):
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             fields = []
             iter = self.model2.get_iter_first()
             while iter:
@@ -182,6 +186,7 @@ class WinImport(WinCSV):
         # TODO: make it works with references
         skip = self.csv_skip.get_value_as_int()
         encoding = self.get_encoding()
+        locale_format = self.csv_locale.get_active()
         reader = csv.reader(
             open(fname, 'r', encoding=encoding),
             quotechar=self.get_quotechar(),
@@ -190,7 +195,20 @@ class WinImport(WinCSV):
         for i, line in enumerate(reader):
             if i < skip or not line:
                 continue
-            data.append([x for x in line])
+            row = []
+            for field, val in zip(fields, line):
+                if locale_format and val:
+                    type_ = self.fields_data[field]['type']
+                    if type_ in ['integer', 'biginteger']:
+                        val = locale.atoi(val)
+                    elif type_ == 'float':
+                        val = locale.atof(val)
+                    elif type_ == 'numeric':
+                        val = locale.atof(val, Decimal)
+                    elif type_ in ['date', 'datetime']:
+                        val = date_parse(val, common.date_format())
+                row.append(val)
+            data.append(row)
         try:
             count = RPCExecute(
                 'model', self.model, 'import_data', fields, data,
