@@ -1,8 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-import gtk
-
-from tryton.gui.main import Main
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import tryton.common as common
 
 
@@ -19,7 +16,8 @@ class NoModal(object):
             focus = focus.get_parent()
 
     def register(self):
-        main = Main.get_main()
+        from tryton.gui.main import Main
+        main = Main()
         self.page = main.get_page()
         if not self.page:
             self.page = main
@@ -27,13 +25,19 @@ class NoModal(object):
         self.sensible_widget.props.sensitive = False
 
     def destroy(self):
-        self.page.dialogs.remove(self)
-        # Test if the parent is not already destroyed
-        if self.parent not in gtk.window_list_toplevels():
+        if not self.page:
             return
+        self.page.dialogs.remove(self)
         self.parent.present()
         self.sensible_widget.props.sensitive = True
         for focus in self.parent_focus:
             if focus and focus.is_ancestor(self.parent):
                 focus.grab_focus()
                 break
+
+    def default_size(self):
+        from tryton.gui.main import Main
+        main = Main()
+        allocation = main.window.get_allocation()
+        width, height = allocation.width, allocation.height
+        return max(width - 150, 0), max(height - 150, 0)
