@@ -1,32 +1,32 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-import gtk
-import gobject
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import gettext
-from interface import WidgetInterface
+
+from gi.repository import Gdk, GLib, Gtk
+
+from .widget import Widget
 import tryton.common as common
 from tryton.gui.window.view_form.screen import Screen
 from tryton.gui.window.win_search import WinSearch
 from tryton.gui.window.win_form import WinForm
-from tryton.config import CONFIG
 from tryton.common.popup_menu import populate
-from tryton.common import RPCExecute, RPCException
 from tryton.common.completion import get_completion, update_completion
-from tryton.common.entry_position import manage_entry_position
+from tryton.common.entry_position import reset_position
+from tryton.common.domain_parser import quote
 
 _ = gettext.gettext
 
 
-class Many2One(WidgetInterface):
+class Many2One(Widget):
 
-    def __init__(self, field_name, model_name, attrs=None):
-        super(Many2One, self).__init__(field_name, model_name, attrs=attrs)
+    def __init__(self, view, attrs):
+        super(Many2One, self).__init__(view, attrs)
 
-        self.widget = gtk.HBox(spacing=0)
+        self.widget = Gtk.HBox(spacing=0)
         self.widget.set_property('sensitive', True)
 
-        self.wid_text = gtk.Entry()
-        self.wid_text.set_property('width-chars', 13)
+        self.wid_text = self.mnemonic_widget = Gtk.Entry()
+        self.wid_text.set_property('width-chars', self.default_width_chars)
         self.wid_text.set_property('activates_default', True)
         self.wid_text.connect('key-press-event', self.send_modified)
         self.wid_text.connect('key_press_event', self.sig_key_press)
@@ -34,80 +34,47 @@ class Many2One(WidgetInterface):
         self.wid_text.connect('focus-out-event',
             lambda x, y: self._focus_out())
         self.wid_text.connect('changed', self.sig_changed)
-        manage_entry_position(self.wid_text)
         self.changed = True
         self.focus_out = True
 
         if int(self.attrs.get('completion', 1)):
-            self.wid_completion = get_completion()
-            self.wid_completion.connect('match-selected',
-                self._completion_match_selected)
-            self.wid_completion.connect('action-activated',
-                self._completion_action_activated)
-            self.wid_text.set_completion(self.wid_completion)
             self.wid_text.connect('changed', self._update_completion)
-        else:
-            self.wid_completion = None
+        self.wid_completion = None
 
-        self.but_open = gtk.Button()
-        img_find = gtk.Image()
-        img_find.set_from_stock('tryton-find', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.but_open.set_image(img_find)
-        self.but_open.set_relief(gtk.RELIEF_NONE)
-        self.but_open.connect('clicked', self.sig_edit)
-        self.but_open.set_alignment(0.5, 0.5)
+        self.wid_text.connect('icon-press', self.sig_edit)
 
-        self.but_new = gtk.Button()
-        img_new = gtk.Image()
-        img_new.set_from_stock('tryton-new', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.but_new.set_image(img_new)
-        self.but_new.set_relief(gtk.RELIEF_NONE)
-        self.but_new.connect('clicked', self.sig_new)
-        self.but_new.set_alignment(0.5, 0.5)
-
-        self.widget.pack_end(self.but_new, expand=False, fill=False)
-        self.widget.pack_end(self.but_open, expand=False, fill=False)
-        self.widget.pack_end(self.wid_text, expand=True, fill=True)
-
-        self.widget.set_focus_chain([self.wid_text])
-
-        self.tooltips = common.Tooltips()
-        self.tooltips.set_tip(self.but_new, _('Create a new record <F3>'))
-        self.tooltips.set_tip(self.but_open, _('Open a record <F2>'))
-        self.tooltips.enable()
+        self.widget.pack_end(self.wid_text, expand=True, fill=True, padding=0)
 
         self._readonly = False
 
-    def grab_focus(self):
-        return self.wid_text.grab_focus()
-
     def get_model(self):
         return self.attrs['relation']
 
     def _readonly_set(self, value):
         self._readonly = value
         self._set_button_sensitive()
-        if value:
-            self.widget.set_focus_chain([])
-        else:
-            self.widget.set_focus_chain([self.wid_text])
 
     def _set_button_sensitive(self):
+        self.wid_text.set_editable(not self._readonly)
+        self.wid_text.set_icon_sensitive(
+            Gtk.EntryIconPosition.PRIMARY, self.read_access)
+        self.wid_text.set_icon_sensitive(
+            Gtk.EntryIconPosition.SECONDARY, not self._readonly)
+
+    def get_access(self, type_):
         model = self.get_model()
         if model:
-            access = common.MODELACCESS[model]
+            return common.MODELACCESS[model][type_]
         else:
-            access = {
-                'create': True,
-                'read': True,
-                }
-        self.wid_text.set_editable(not self._readonly)
-        self.but_new.set_sensitive(bool(
-                not self._readonly
-                and self.attrs.get('create', True)
-                and access['create']))
-        self.but_open.set_sensitive(bool(
-                access['read']))
+            return True
+
+    @property
+    def read_access(self):
+        return self.get_access('read')
+
+    @property
+    def create_access(self):
+        return self.attrs.get('create', True) and self.get_access('create')
 
     @property
     def modified(self):
@@ -116,9 +83,6 @@ class Many2One(WidgetInterface):
             return self.field.get_client(self.record) != value
         return False
 
-    def _color_widget(self):
-        return self.wid_text
-
     @staticmethod
     def has_target(value):
         return value is not None
@@ -150,28 +114,9 @@ class Many2One(WidgetInterface):
                         or self.field.get_state_attrs(
                             self.record)['required'])):
                 domain = self.field.domain_get(self.record)
-                context = self.field.context_get(self.record)
-                self.wid_text.grab_focus()
-
-                try:
-                    if self.wid_text.get_text():
-                        dom = [('rec_name', 'ilike',
-                            '%' + self.wid_text.get_text() + '%'),
-                            domain]
-                    else:
-                        dom = domain
-                    ids = RPCExecute('model', model, 'search', dom, 0,
-                        CONFIG['client.limit'], None, context=context)
-                except RPCException:
-                    self.focus_out = True
-                    self.changed = True
-                    return
-                if len(ids) == 1:
-                    self.field.set_client(self.record,
-                        self.value_from_id(ids[0]), force_change=True)
-                    self.focus_out = True
-                    self.changed = True
-                    return
+                context = self.field.get_search_context(self.record)
+                order = self.field.get_search_order(self.record)
+                text = self.wid_text.get_text()
 
                 def callback(result):
                     if result:
@@ -182,11 +127,17 @@ class Many2One(WidgetInterface):
                     self.focus_out = True
                     self.changed = True
 
-                WinSearch(model, callback, sel_multi=False,
-                    ids=ids, context=context, domain=domain,
+                win = WinSearch(model, callback, sel_multi=False,
+                    context=context, domain=domain, order=order,
                     view_ids=self.attrs.get('view_ids', '').split(','),
                     views_preload=self.attrs.get('views', {}),
-                    new=self.but_new.get_property('sensitive'))
+                    new=self.create_access,
+                    title=self.attrs.get('string'))
+                win.screen.search_filter(quote(text))
+                if len(win.screen.group) == 1:
+                    win.response(None, Gtk.ResponseType.OK)
+                else:
+                    win.show()
                 return
         self.focus_out = True
         self.changed = True
@@ -194,9 +145,11 @@ class Many2One(WidgetInterface):
 
     def get_screen(self):
         domain = self.field.domain_get(self.record)
-        context = self.field.context_get(self.record)
+        context = self.field.get_context(self.record)
+        # Remove first tree view as mode is form only
+        view_ids = self.attrs.get('view_ids', '').split(',')[1:]
         return Screen(self.get_model(), domain=domain, context=context,
-            mode=['form'], view_ids=self.attrs.get('view_ids', '').split(','),
+            mode=['form'], view_ids=view_ids,
             views_preload=self.attrs.get('views', {}), readonly=self._readonly,
             exclude_field=self.attrs.get('relation_field'))
 
@@ -213,20 +166,29 @@ class Many2One(WidgetInterface):
                     self.value_from_id(screen.current_record.id,
                         screen.current_record.rec_name()))
             self.focus_out = True
-        WinForm(screen, callback, new=True, save_current=True)
+        WinForm(screen, callback, new=True, save_current=True,
+            title=self.attrs.get('string'), rec_name=self.wid_text.get_text())
 
-    def sig_edit(self, *args):
+    def sig_edit(self, entry=None, icon_pos=None, *args):
         model = self.get_model()
         if not model or not common.MODELACCESS[model]['read']:
             return
         if not self.focus_out or not self.field:
             return
         self.changed = False
-        value = self.field.get(self.record)
-        model = self.get_model()
-
         self.focus_out = False
-        if model and self.has_target(value):
+        value = self.field.get(self.record)
+
+        if (icon_pos == Gtk.EntryIconPosition.SECONDARY
+                and not self._readonly
+                and self.has_target(value)):
+            self.field.set_client(self.record, self.value_from_id(None, ''))
+            self.wid_text.set_text('')
+            self.changed = True
+            self.focus_out = True
+            return
+
+        if self.has_target(value):
             screen = self.get_screen()
             screen.load([self.id_from_value(self.field.get(self.record))])
 
@@ -238,31 +200,14 @@ class Many2One(WidgetInterface):
                         force_change=True)
                 self.focus_out = True
                 self.changed = True
-            WinForm(screen, callback, save_current=True)
+            WinForm(screen, callback, save_current=True,
+                title=self.attrs.get('string'))
             return
-        elif model and not self._readonly:
+        if not self._readonly:
             domain = self.field.domain_get(self.record)
-            context = self.field.context_get(self.record)
-            self.wid_text.grab_focus()
-
-            try:
-                if self.wid_text.get_text():
-                    dom = [('rec_name', 'ilike',
-                        '%' + self.wid_text.get_text() + '%'),
-                        domain]
-                else:
-                    dom = domain
-                ids = RPCExecute('model', model, 'search', dom, 0,
-                    CONFIG['client.limit'], None, context=context)
-            except RPCException:
-                self.focus_out = True
-                self.changed = True
-                return False
-            if len(ids) == 1:
-                self.field.set_client(self.record, self.value_from_id(ids[0]),
-                    force_change=True)
-                self.focus_out = True
-                return True
+            context = self.field.get_search_context(self.record)
+            order = self.field.get_search_order(self.record)
+            text = self.wid_text.get_text()
 
             def callback(result):
                 if result:
@@ -270,28 +215,28 @@ class Many2One(WidgetInterface):
                         self.value_from_id(*result[0]), force_change=True)
                 self.focus_out = True
                 self.changed = True
-            WinSearch(model, callback, sel_multi=False,
-                ids=ids, context=context, domain=domain,
+            win = WinSearch(model, callback, sel_multi=False,
+                context=context, domain=domain, order=order,
                 view_ids=self.attrs.get('view_ids', '').split(','),
                 views_preload=self.attrs.get('views', {}),
-                new=self.but_new.get_property('sensitive'))
+                new=self.create_access, title=self.attrs.get('string'))
+            win.screen.search_filter(quote(text))
+            win.show()
             return
         self.focus_out = True
         self.changed = True
-        return
 
     def sig_key_press(self, widget, event, *args):
         editable = self.wid_text.get_editable()
-        activate_keys = [gtk.keysyms.Tab, gtk.keysyms.ISO_Left_Tab]
+        activate_keys = [Gdk.KEY_Tab, Gdk.KEY_ISO_Left_Tab]
         if not self.wid_completion:
-            activate_keys.append(gtk.keysyms.Return)
-        if (event.keyval == gtk.keysyms.F3
+            activate_keys.append(Gdk.KEY_Return)
+        if (event.keyval == Gdk.KEY_F3
                 and editable
-                and self.but_new.get_property('sensitive')):
+                and self.create_access):
             self.sig_new(widget, event)
             return True
-        elif (event.keyval == gtk.keysyms.F2
-                and self.but_open.get_property('sensitive')):
+        elif event.keyval == Gdk.KEY_F2 and self.read_access:
             self.sig_edit(widget)
             return True
         elif (event.keyval in activate_keys
@@ -299,8 +244,7 @@ class Many2One(WidgetInterface):
             self.sig_activate()
         elif (self.has_target(self.field.get(self.record))
                 and editable
-                and event.keyval in (gtk.keysyms.Delete,
-                    gtk.keysyms.BackSpace)):
+                and event.keyval in [Gdk.KEY_Delete, Gdk.KEY_BackSpace]):
             self.wid_text.set_text('')
         return False
 
@@ -322,59 +266,80 @@ class Many2One(WidgetInterface):
                     # Restore text and position after display
                     self.wid_text.set_text(text)
                     self.wid_text.set_position(position)
-            gobject.idle_add(clean)
+            GLib.idle_add(clean)
         return False
 
     def get_value(self):
         return self.wid_text.get_text()
 
-    def set_value(self, record, field):
-        if field.get_client(record) != self.wid_text.get_text():
-            field.set_client(record, self.value_from_id(None, ''))
+    def set_value(self):
+        if self.field.get_client(self.record) != self.wid_text.get_text():
+            self.field.set_client(self.record, self.value_from_id(None, ''))
             self.wid_text.set_text('')
 
     def set_text(self, value):
         if not value:
             value = ''
         self.wid_text.set_text(value)
-        self.wid_text.set_position(len(value))
+        reset_position(self.wid_text)
 
-    def display(self, record, field):
+    def display(self):
         self.changed = False
-        super(Many2One, self).display(record, field)
+        super(Many2One, self).display()
 
         self._set_button_sensitive()
+        self._set_completion()
 
-        if not field:
-            self.wid_text.set_text('')
-            self.wid_text.set_position(0)
+        if not self.field:
+            self.set_text(None)
             self.changed = True
             return False
-        img = self.but_open.get_image()
-        current_stock = img.get_stock()[0]
-        self.set_text(field.get_client(record))
-        value = field.get(record)
-        if self.has_target(value) and current_stock != 'tryton-open':
-            img.set_from_stock('tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR)
-            self.tooltips.set_tip(self.but_open, _('Open a record <F2>'))
-        elif not self.has_target(value) and current_stock != 'tryton-find':
-            img.set_from_stock('tryton-find', gtk.ICON_SIZE_SMALL_TOOLBAR)
-            self.tooltips.set_tip(self.but_open, _('Search a record <F2>'))
+        self.set_text(self.field.get_client(self.record))
+        if self.has_target(self.field.get(self.record)):
+            icon1, tooltip1 = 'tryton-open', _('Open the record <F2>')
+            icon2, tooltip2 = 'tryton-clear', _('Clear the field <Del>')
+        else:
+            icon1, tooltip1 = None, ''
+            icon2, tooltip2 = 'tryton-search', _('Search a record <F2>')
+        if not self.wid_text.get_editable():
+            icon2, tooltip2 = None, ''
+        for pos, icon, tooltip in [
+                (Gtk.EntryIconPosition.PRIMARY, icon1, tooltip1),
+                (Gtk.EntryIconPosition.SECONDARY, icon2, tooltip2)]:
+            if icon:
+                pixbuf = common.IconFactory.get_pixbuf(
+                    icon, Gtk.IconSize.MENU)
+            else:
+                pixbuf = None
+            self.wid_text.set_icon_from_pixbuf(pos, pixbuf)
+            self.wid_text.set_icon_tooltip_text(pos, tooltip)
         self.changed = True
 
     def _populate_popup(self, widget, menu):
         value = self.field.get(self.record)
         if self.has_target(value):
-            # Delay filling of popup as it can take time
-            gobject.idle_add(populate, menu, self.get_model(),
-                self.id_from_value(value), '', self.field)
+            populate(
+                menu, self.get_model(), self.id_from_value(value),
+                '', self.field)
         return True
 
+    def _set_completion(self):
+        if not int(self.attrs.get('completion', 1)):
+            return
+        self.wid_completion = get_completion(
+            search=self.read_access,
+            create=self.create_access)
+        self.wid_completion.connect('match-selected',
+            self._completion_match_selected)
+        self.wid_completion.connect('action-activated',
+            self._completion_action_activated)
+        self.wid_text.set_completion(self.wid_completion)
+
     def _completion_match_selected(self, completion, model, iter_):
         rec_name, record_id = model.get(iter_, 0, 1)
         # GTK on win32 doesn't like synchronous call to set_client
         # because it triggers a display which reset the completion
-        gobject.idle_add(self.field.set_client, self.record,
+        GLib.idle_add(self.field.set_client, self.record,
             self.value_from_id(record_id, rec_name), True)
 
         completion_model = self.wid_completion.get_model()
